from ntrprtr.action.ActionBase import ActionBase

class DecimalAction(ActionBase):
    def __init__(self):
        super().__init__()

    def process(self, action, _bytes):
        result = ""
        endianess = action["endianess"]
        if(endianess == "big"):
            result = self._cnvrtr.hexToDec(_bytes.hex())
        elif(endianess == "little"):
            result = self.__hexToLittleEndianToDec(_bytes)
        return result

    def __hexToLittleEndianToDec(self, byteArr):
        le = self._cnvrtr.toLittleEndian(byteArr.hex(" "))
        return str(self._cnvrtr.hexToDec(le))


from cnvrtr.Converter import Converter
from ntrprtr.action.ActionType import ActionType



class Printer():
    def __init__(self) -> None:
        self._cnvrtr = Converter(nonAsciiPlaceholder=".")

    def print(self, result):
        for r in result:
            print("")
            print("--> " + r[1])
            print("    --------------")
            if(r[2] == ActionType.HEXDUMP):
                print("    Hexdump")
                print("    --------------")
                print("")
                dump = r[4].split("\n")
                for l in dump:
                    print("    " + l)
            elif(r[2] == ActionType.BITEQUALS):
                print("    " + r[3].hex(" ").upper())
                print("    --------------")
                dump = r[4].split("\n")
                for l in dump:
                    print("    " + l)
            else: 
                print("    " + r[3].hex(" ").upper())
                print("    --------------")
                print("    " + r[4])
           


from ntrprtr.action.ActionBase import ActionBase

class BitEqualsAction(ActionBase):
    def __init__(self):
        super().__init__()

    def process(self, action, _bytes):
        bits = ""
        result = action["noMatch"]
        
        endianess = action["endianess"]
        if(endianess == "big"):
            bits = self._cnvrtr.hexToBin(_bytes.hex())
        elif(endianess == "little"):
            bits = self.__hexToLittleEndianToBin(_bytes)
        
        # Always fill up zeros to represent all given bytes, even if 
        # they are zero
        bits = bits.rjust(len(_bytes)*8, "0")

        for i in range(0, len(action["cmp"])):
            if(bits == action["cmp"][i]["value"]):
                result = " ".join(bits[i:i+4] for i in range(0, len(bits), 4)) + "\n--------------\n" + action["cmp"][i]["description"]

        return result

    def __hexToLittleEndianToBin(self, byteArr):
        le = self._cnvrtr.toLittleEndian(byteArr.hex(" "))
        return str(self._cnvrtr.hexToBin(le))


from ntrprtr.action.ActionBase import ActionBase

class DOSDateAction(ActionBase):
    def __init__(self):
        super().__init__()

    def process(self, action, _bytes):
        r = ""
        c = self._cnvrtr
        hexValues = _bytes.hex(" ")

        endianess = action["endianess"]
        if(endianess == "big"):
            r = self._cnvrtr.hexToBin(_bytes.hex()).rjust(16, "0")
        elif(endianess == "little"):
            r = c.hexToBin(c.toLittleEndian(hexValues)).rjust(16, "0")
        
        yearBits = [r[i:i + 7] for i in range(0, 7, 7)][0]
        monthBits = [r[i:i + 4] for i in range(7, 11, 4)][0]
        dayBits = [r[i:i + 5] for i in range(11, 16, 5)][0]

        return str(c.binToDec(dayBits)) + "." + str(c.binToDec(monthBits)) + "." +  str(c.binToDec(yearBits)+1980)


class ActionType():
    DECIMAL = "decimal"
    ASCII = "ascii"
    EQUALS = "equals"
    BITEQUALS = "bitequals"
    BINARY = "binary"
    HEXDUMP = "hexdump"
    DOSDATE = "dosdate"
    DOSTIME = "dostime"
    UNICODE = "unicode"


from ntrprtr.action.ActionType import ActionType
from ntrprtr.action.DOSDateAction import DOSDateAction
from ntrprtr.action.DOSTimeAction import DOSTimeAction
from ntrprtr.action.DecimalAction import DecimalAction
from ntrprtr.action.AsciiAction import AsciiAction
from ntrprtr.action.BinaryAction import BinaryAction
from ntrprtr.action.EqualsAction import EqualsAction
from ntrprtr.action.BitEqualsAction import BitEqualsAction
from ntrprtr.action.HexdumpAction import HexdumpAction
from ntrprtr.action.UnicodeAction import UnicodeAction

class ByteInterpreter():
    def __init__(self, bytes, config) -> None:
        self._bytes = bytes
        self._config = config        

    def interpret(self):
        result = []
        for c in self._config:
            b = bytearray()
            amount = c["end"] - c["start"] + 1
            subBytes = [self._bytes[i:i + amount] for i in range(c["start"], c["end"]+1, amount)][0]
            b.extend(subBytes)
            if(c.get("action") != None):
                actionResult = self.__processActions(c["action"], b)
                result.append((c["name"], c["description"], c["action"]["type"], b, actionResult))
            else:
                actionResult = ""
                result.append((c["name"], c["description"], "None", b, actionResult)) 
        return result

    def __processActions(self, action, b):
        result = ""
        type_ = action["type"]
        if(type_ == ActionType.DECIMAL):
            result = DecimalAction().process(action, b)
        elif(type_ == ActionType.ASCII):
            result = AsciiAction().process(action, b)
        elif(type_ == ActionType.EQUALS):
            result = EqualsAction().process(action, b)
        elif(type_ == ActionType.BITEQUALS):
            result = BitEqualsAction().process(action, b)
        elif(type_ == ActionType.BINARY):
            result = BinaryAction().process(action, b)
        elif(type_ == ActionType.HEXDUMP):
            result = HexdumpAction().process(action, b)
        elif(type_ == ActionType.DOSDATE):
            result = DOSDateAction().process(action, b)
        elif(type_ == ActionType.DOSTIME):
            result = DOSTimeAction().process(action, b)
        elif(type_ == ActionType.UNICODE):
            result = UnicodeAction().process(action, b)

        return result

    


import sys
import json

import argparse


def main(args_=None):
    """The main routine."""
    if args_ is None:
        args_ = sys.argv[1:]

    parser = argparse.ArgumentParser()
    parser.add_argument("--obj", "-o", type=int, required=True, help="Create a config with the given number of objects")
    parser.add_argument("--name", "-n", type=str, default="config.json", help="Name of the config file")
    args = parser.parse_args()

    config = []

    for i in range(0, args.obj):
        o = { "name": "Name", "description": "desc", "start": 0, "end": 0, "action": { "type": "type" }}
        config.append(o)

    j = json.dumps(config, indent=4)

    with open(args.name, "w")as f:
        f.write(j)



if __name__ == "__main__":
    sys.exit(main())



from ntrprtr.action.ActionBase import ActionBase

class BinaryAction(ActionBase):
    def __init__(self):
        super().__init__()

    def process(self, action, _bytes):
        result = ""

        endianess = action["endianess"]
        if(endianess == "big"):
            result = self._cnvrtr.hexToBin(_bytes.hex())
        elif(endianess == "little"):
            result = self.__hexToLittleEndianToBin(_bytes)
        
        # Always fill up zeros to represent all given bytes, even if 
        # they are zero
        result = result.rjust(len(_bytes)*8, "0")
        # For better readability add a space after 4 bit
        result = " ".join(result[i:i+4] for i in range(0, len(result), 4))
        return result

    def __hexToLittleEndianToBin(self, byteArr):
        le = self._cnvrtr.toLittleEndian(byteArr.hex(" "))
        return str(self._cnvrtr.hexToBin(le))


from ntrprtr.action.ActionBase import ActionBase

class AsciiAction(ActionBase):
    def __init__(self):
        super().__init__()

    def process(self, action, _bytes):
        self._cnvrtr._nonAsciiPlaceholder = action["nonAsciiPlaceholder"]
        return self._cnvrtr.hexToAsciiString(_bytes.hex())


from ntrprtr.action.ActionBase import ActionBase

class UnicodeAction(ActionBase):
    def __init__(self):
        super().__init__()

    def process(self, action, _bytes):
        return _bytes.decode("utf-8")


from ntrprtr.action.ActionBase import ActionBase

class HexdumpAction(ActionBase):
    def __init__(self):
        super().__init__()

    def process(self, action, _bytes):
        offset_ = 0
        result = self.__createHexHeader()
        splittedBytes = [_bytes[i:i + 16] for i in range(0, len(_bytes), 16)]
        for b_ in splittedBytes:
            result += self.__createHexRow(offset_, b_.hex(" "), b_, action["nonAsciiPlaceholder"])
            offset_ += 16  
        return result

    def __createHexHeader(self):
        result = ""
        h = "{:8}   {:2} {:2} {:2} {:2} {:2} {:2} {:2} {:2} {:2} {:2} {:2} {:2} {:2} {:2} {:2} {:2}    {:16}".format("  Offset", "00", "01", "02", "03", "04", "05",
                                                                                                                            "06", "07", "08", "09", "0A", "0B",
                                                                                                                            "0C", "0D", "0E", "0F", "ASCII")
        result += h + "\n"
        result += ("-"*8) + "   " + ("-"*47) + "    " + ("-"*16) +" \n"
        return result

    def __createHexRow(self, offset, hex_, bytes_, placeholder):
        formatStr = "{:8}   {:47}    {:16}"
        asc = "".join(chr(v) if (v >= 32 and v <= 126) else placeholder for v in bytes_)
        r = formatStr.format(offset, hex_.upper(), asc)
        return r + " \n"




import binascii

class TestFileCreator():
    def __init__(self) -> None:
       pass

    def createFile(self, inputPath, outputPath):
        with open(inputPath) as inFile, open(outputPath, "wb") as outFile:
            for line in inFile:
                outFile.write(binascii.unhexlify(''.join(line.split())))


from ntrprtr.action.ActionBase import ActionBase

class DOSTimeAction(ActionBase):
    def __init__(self):
        super().__init__()

    def process(self, action, _bytes):
        r = ""
        c = self._cnvrtr
        hexValues = _bytes.hex(" ")
        
        endianess = action["endianess"]
        if(endianess == "big"):
            r = c.hexToBin(_bytes.hex()).rjust(16, "0")
        elif(endianess == "little"):
            r = c.hexToBin(c.toLittleEndian(hexValues)).rjust(16, "0")   

        hourBits = [r[i:i + 5] for i in range(0, 5, 5)][0]
        minuteBits = [r[i:i + 6] for i in range(5, 11, 6)][0]
        secondBits = [r[i:i + 5] for i in range(11, 16, 5)][0]

        return str(c.binToDec(hourBits)) + ":" + str(c.binToDec(minuteBits)) + ":" +  str(c.binToDec(secondBits)*2)



from ntrprtr.action.ActionBase import ActionBase

class EqualsAction(ActionBase):
    def __init__(self):
        super().__init__()

    def process(self, action, _bytes):
        b = ""
        result = action["noMatch"]

        endianess = action["endianess"]

        if(endianess == "big"):
            b = _bytes.hex()
        elif(endianess == "little"):
            b = self._cnvrtr.toLittleEndian(_bytes.hex(" "))


        for i in range(0, len(action["cmp"])):
            if(b == action["cmp"][i]["value"].lower()):
                result = action["cmp"][i]["description"]
        return result